-- VHDL Entity Board.FPGA_dcMotorController.symbol
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 15:24:24 03.09.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
   USE ieee.std_logic_1164.all;
   USE ieee.numeric_std.all;

ENTITY FPGA_dcMotorController IS
   GENERIC( 
      ioBitNb   : positive := 20;
      jtagBitNb : positive := 3
   );
   PORT( 
      clock   : IN     std_ulogic;
      reset_n : IN     std_uLogic;
      sCl     : IN     std_logic;
      jtagIo  : OUT    std_ulogic_vector (1 TO jtagBitNb);
      I_O     : INOUT  std_logic_vector (1 TO ioBitNb);
      sDa     : INOUT  std_logic
   );

-- Declarations

END FPGA_dcMotorController ;





-- VHDL Entity DcMotor.dcMotorController.symbol
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 15:24:24 03.09.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY dcMotorController IS
   GENERIC( 
      i2cBitNb               : positive := 10;
      speedBitNb             : positive := 4;
      prescalerBitNb         : positive := 16;
      kartBaseAddress        : positive := 16#51#;
      dcBaseAddress          : natural  := 0;
      orientationBaseAddress : natural  := 8;
      hwOrientationBitNb     : positive := 3;
      testOutBitNb           : positive := 8
   );
   PORT( 
      clock    : IN     std_ulogic;
      reset    : IN     std_ulogic;
      sClIn    : IN     std_uLogic;
      sDaIn    : IN     std_uLogic;
      forwards : OUT    std_ulogic;
      pwm      : OUT    std_ulogic;
      sDaOut   : OUT    std_uLogic;
      testOut  : OUT    std_ulogic_vector (1 TO testOutBitNb)
   );

-- Declarations

END dcMotorController ;





-- VHDL Entity DcMotor.dcMotorPrescaler.symbol
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 15:24:24 03.09.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY dcMotorPrescaler IS
   GENERIC( 
      prescalerBitNb : positive := 16
   );
   PORT( 
      clock     : IN     std_ulogic;
      reset     : IN     std_ulogic;
      pwmEn     : OUT    std_ulogic;
      prescaler : IN     unsigned (prescalerBitNb-1 DOWNTO 0)
   );

-- Declarations

END dcMotorPrescaler ;





ARCHITECTURE RTL OF dcMotorPrescaler IS

  signal prescalerCounter: unsigned(prescaler'range);
  signal prescalerDone: std_ulogic;

BEGIN

  divideClock: process(reset, clock)
  begin
    if reset = '1' then
      prescalerCounter <= (others => '0');
    elsif rising_edge(clock) then
      if prescalerDone = '1' then
        prescalerCounter <= (others => '0');
      else
        prescalerCounter <= prescalerCounter + 1;
      end if;
    end if;
  end process divideClock;

  prescalerDone <= '1' when prescalerCounter+1 >= prescaler
    else '0';
  pwmEn <= prescalerDone;

END ARCHITECTURE RTL;




-- VHDL Entity I2C.i2cTransceiver.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 11:18:30 19.05.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY i2cTransceiver IS
   GENERIC( 
      dataBitNb : positive := 10
   );
   PORT( 
      sDaIn        : IN     std_ulogic;
      clock        : IN     std_ulogic;
      reset        : IN     std_ulogic;
      dataIn       : OUT    std_ulogic_vector (dataBitNb-2-1 DOWNTO 0);
      dataValid    : OUT    std_ulogic;
      sCl          : IN     std_ulogic;
      registerAddr : OUT    unsigned (dataBitNb-2-1 DOWNTO 0);
      writeData    : OUT    std_ulogic;
      sDaOut       : OUT    std_ulogic;
      dataOut      : IN     std_ulogic_vector (dataBitNb-2-1 DOWNTO 0);
      chipAddr     : OUT    unsigned (dataBitNb-3-1 DOWNTO 0);
      isSelected   : IN     std_ulogic
   );

-- Declarations

END i2cTransceiver ;





PACKAGE CommonLib IS

  function requiredBitNb (val : integer) return integer;

END CommonLib;




PACKAGE BODY CommonLib IS

  function requiredBitNb (val : integer) return integer is
    variable powerOfTwo, bitNb : integer;
  begin
    powerOfTwo := 1;
    bitNb := 0;
    while powerOfTwo <= val loop
      powerOfTwo := 2 * powerOfTwo;
      bitNb := bitNb + 1;
    end loop;
    return bitNb;
  end requiredBitNb;

END CommonLib;




library Common;
  use Common.CommonLib.all;

architecture RTL of i2cTransceiver is
                                                                     -- receiver
  signal clDelayed      : std_uLogic;
  signal daDelayed      : std_uLogic;
  signal clRising       : std_uLogic;
  signal clFalling      : std_uLogic;
  signal daRising       : std_uLogic;
  signal daFalling      : std_uLogic;
  signal startCondition : std_uLogic;
  signal stopCondition  : std_uLogic;
  signal bitCounter     : unsigned(requiredBitNb(dataBitNb)-1 downto 0);
  signal inputShiftReg  : std_ulogic_vector(dataBitNb-2 downto 0);
  signal clRisingDelayed: std_uLogic;
  signal endOfWord      : std_uLogic;
  signal i2cByteValid   : std_uLogic;
  signal i2cByte        : std_ulogic_vector(dataIn'range);
                                                                          -- ack
  signal ackMoment      : std_uLogic;
  signal sendAck        : std_uLogic;
                                                                      -- decoder
  type   rxStateType    is (idle, readChipAddress, readRegAddress,
                            incAddress1, incAddress);
  signal rxState        : rxStateType;
  signal chipAddress    : unsigned(chipAddr'range);
  signal registerAddress: unsigned(registerAddr'range);
  signal writeMode      : std_uLogic;

  signal loadOutShiftReg: std_uLogic;
  signal enSdaOut       : std_uLogic;
  signal outputShiftReg : std_ulogic_vector(dataBitNb-2-1 downto 0);

begin
  --============================================================================
                                             -- start, stop and other conditions
  delayInputs: process(reset, clock)
  begin
    if reset = '1' then
      clDelayed <= '1';
      daDelayed <= '1';
      clRisingDelayed <= '0';
    elsif rising_edge(clock) then
      clDelayed <= sCl;
      daDelayed <= sDaIn;
      clRisingDelayed <= clRising;
    end if;
  end process delayInputs;

  clRising <= '1' when (sCl = '1') and (clDelayed = '0') else '0';
  clFalling <= '1' when (sCl = '0') and (clDelayed = '1') else '0';
  daRising <= '1' when (sDaIn = '1') and (daDelayed = '0') else '0';
  daFalling <= '1' when (sDaIn = '0') and (daDelayed = '1') else '0';
  startCondition <= '1' when (daFalling = '1') and (sCl = '1') else '0';
  stopCondition <= '1' when (daRising = '1') and (sCl = '1') else '0';

  ------------------------------------------------------------------------------
                                                                  -- bit counter
  countBitNb: process(reset, clock)
  begin
    if reset = '1' then
      bitCounter <= (others => '0');
    elsif rising_edge(clock) then
      if startCondition = '1' then
        bitCounter <= (others => '0');
      elsif stopCondition = '1' then
        bitCounter <= (others => '0');
      elsif clFalling = '1' then
        if bitCounter < dataBitNb-1 then
          bitCounter <= bitCounter + 1;
        else
          bitCounter <= to_unsigned(1, bitCounter'length);
        end if;
      end if;
    end if;
  end process countBitNb;

  endOfWord <= '1' when (clRisingDelayed = '1') and (bitCounter = dataBitNb-1)
    else '0';

  --============================================================================
                                                         -- input shift register
  shiftInputBits: process(reset, clock)
  begin
    if reset = '1' then
      inputShiftReg <= (others => '0');
    elsif rising_edge(clock) then
      if clRising = '1' then
        inputShiftReg(inputShiftReg'high downto 1) <= inputShiftReg(inputShiftReg'high-1 downto 0);
        inputShiftReg(0) <= sDaIn;
      end if;
    end if;
  end process shiftInputBits;

  ------------------------------------------------------------------------------
                                                -- read byte from shift register
  i2cByteValid <= '1' when (bitCounter = i2cByte'length) and (clFalling = '1')
    else '0';
  i2cByte <= inputShiftReg(i2cByte'range);

  storeByte: process(reset, clock)
  begin
    if reset = '1' then
      dataIn <= (others => '0');
    elsif rising_edge(clock) then
      if i2cByteValid = '1' then
        dataIn <= i2cByte;
      end if;
    end if;
  end process storeByte;

  --============================================================================
                                                           -- address extraction
  rxFsm: process(reset, clock)
  begin
    if reset = '1' then
      rxState <= idle;
    elsif rising_edge(clock) then
      if stopCondition = '1' then
        rxState <= idle;
      elsif startCondition = '1' then
        rxState <= readChipAddress;
      elsif endOfWord = '1' then
        case rxState is
          when readChipAddress =>
            if writeMode = '1' then
              rxState <= readRegAddress;
            else
              rxState <= incAddress1;
            end if;
          when readRegAddress =>
            rxState <= incAddress1;
          when incAddress1 =>
            rxState <= incAddress;
          when incAddress =>
            if (writeMode = '0') and (sDaIn = '1') then
              rxState <= idle;
            end if;
          when others => null;
        end case;
      end if;
    end if;
  end process rxFsm;

  dataValid <= endOfWord when (rxState = incAddress1) or (rxState = incAddress)
    else '0';

  ------------------------------------------------------------------------------
                                                  -- provide running i2c address
  updateAddress: process(reset, clock)
  begin
    if reset = '1' then
      chipAddress <= (others => '0');
      registerAddress <= (others => '0');
      writeMode <= '0';
    elsif rising_edge(clock) then
      if i2cByteValid = '1' then
        case rxState is
          when readChipAddress =>
            chipAddress <= resize(
              shift_right(unsigned(i2cByte), 1),
              chipAddress'length
            );
            if i2cByte(0) = '0' then -- i2cByte(0)=0 -> master write to slave
              writeMode <= '1';
            else
              writeMode <= '0';
              registerAddress <= registerAddress + 1;
            end if;
          when readRegAddress =>
            registerAddress <= resize(unsigned(i2cByte)-1, registerAddress'length);
          when incAddress1 | incAddress =>
            registerAddress <= registerAddress + 1;
          when others => null;
        end case;
      end if;
    end if;
  end process updateAddress;

  chipAddr <= chipAddress;
  registerAddr <= registerAddress;
  writeData <= writeMode;

  ------------------------------------------------------------------------------
                                                                  -- acknowledge
  findAckMoment: process(reset, clock)
  begin
    if reset = '1' then
      ackMoment <= '0';
      sendAck <= '0';
    elsif rising_edge(clock) then
      if (bitCounter = dataBitNb-2) and (clFalling = '1') then
        ackMoment <= '1';
      elsif clFalling = '1' then
        ackMoment <= '0';
      end if;
      if ackMoment = '0' then
        sendAck <= '0';
      elsif isSelected = '1' then
        if writeMode = '1' then
          sendAck <= '1';
        elsif rxState = readChipAddress then
          sendAck <= '1';
        end if;
      end if;
    end if;
  end process findAckMoment;

  --============================================================================
                                                              -- output controls
  controlAnswering: process(reset, clock)
  begin
    if reset = '1' then
      loadOutShiftReg <= '0';
      enSdaOut <= '0';
    elsif rising_edge(clock) then
      if endOfWord = '1' then
        loadOutShiftReg <= '1';
      elsif clRising = '1' then
        loadOutShiftReg <= '0';
      end if;
      if
        (bitCounter = 1) and
        ( (rxState = incAddress1) or (rxState = incAddress) ) and
        (writeMode = '0')
      then
        if sDaIn /= '0' then  -- let master finish ack pulse
          enSdaOut <= '1';
        end if;
      elsif ( (bitCounter = dataBitNb-2) and (clFalling = '1') ) or (rxState = idle) then
        enSdaOut <= '0';
      end if;
    end if;
  end process controlAnswering;

  ------------------------------------------------------------------------------
                                                        -- output shift register
  shiftOutputBits: process(reset, clock)
  begin
    if reset = '1' then
      outputShiftReg <= (others => '0');
    elsif rising_edge(clock) then
      if loadOutShiftReg = '1' then
        outputShiftReg <= (others => '1');
        outputShiftReg(dataOut'range) <= dataOut;
      elsif clFalling = '1' then
        outputShiftReg(outputShiftReg'high downto 1) <= outputShiftReg(outputShiftReg'high-1 downto 0);
        outputShiftReg(0) <= '0';
      end if;
    end if;
  end process shiftOutputBits;

  ------------------------------------------------------------------------------
                                                        -- reply to read request
  sDaOut <= '0' when sendAck = '1'
    else outputShiftReg(outputShiftReg'high) when enSdaOut = '1'
    else '1';

end RTL;




-- VHDL Entity DcMotor.dcMotorPwm.symbol
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 15:24:24 03.09.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY dcMotorPwm IS
   GENERIC( 
      speedBitNb : positive := 8
   );
   PORT( 
      clock           : IN     std_ulogic;
      reset           : IN     std_ulogic;
      en              : IN     std_ulogic;
      speed           : IN     signed (5-1 DOWNTO 0);           -- (speedBitNb-1:0)
      pwmOut          : OUT    std_ulogic;
      forwards        : OUT    std_ulogic;
      normalDirection : IN     std_ulogic;
      testOut         : OUT    std_ulogic_vector (7 DOWNTO 0)
   );

-- Declarations

END dcMotorPwm ;





ARCHITECTURE RTL OF dcMotorPwm IS

  signal sign: std_ulogic;
  signal speedAmplitude: unsigned(speed'high-1 downto 0);
  signal sawtooth: unsigned(speedAmplitude'range);

BEGIN
  ------------------------------------------------------------------------------
                                                                    -- direction
  sign <= speed(speed'high);
  forwards <= not sign when normalDirection = '1'
    else sign;

  ------------------------------------------------------------------------------
                                                                          -- PWM
  speedAmplitude <= unsigned(speed(speedAmplitude'range)) when sign = '0'
    else unsigned(-speed(speedAmplitude'range));

  storeValues: process(reset, clock)
  begin
    if reset = '1' then
      sawtooth <= (others => '0');
    elsif rising_edge(clock) then
      if en = '1' then
        sawtooth <= sawtooth + 1;
      end if;
    end if;
  end process storeValues;

  pwmOut <= '0' when speedAmplitude = 0
    else '1' when speedAmplitude >= sawtooth
    else '0';

END ARCHITECTURE RTL;




-- VHDL Entity DcMotor.dcMotorRegisters.symbol
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 15:24:24 03.09.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY dcMotorRegisters IS
   GENERIC( 
      busAddressBitNb        : positive := 8;
      busDataBitNb           : positive := 8;
      speedBitNb             : positive := 8;
      prescalerBitNb         : positive := 16;
      baseAddress            : natural  := 0;
      hwOrientationBitNb     : positive := 3;
      orientationBaseAddress : natural  := 8
   );
   PORT( 
      clock         : IN     std_ulogic;
      reset         : IN     std_ulogic;
      busDataIn     : IN     std_ulogic_vector (busDataBitNb-1 DOWNTO 0);
      en            : IN     std_ulogic;
      speed         : OUT    signed (speedBitNb-1 DOWNTO 0);
      busAddress    : IN     unsigned (busAddressBitNb-1 DOWNTO 0);
      prescaler     : OUT    unsigned (prescalerBitNb-1 DOWNTO 0);
      write         : IN     std_ulogic;
      busDataOut    : OUT    std_ulogic_vector (busDataBitNb-1 DOWNTO 0);
      hwOrientation : OUT    std_ulogic_vector (hwOrientationBitNb-1 DOWNTO 0)
   );

-- Declarations

END dcMotorRegisters ;





ARCHITECTURE RTL OF dcMotorRegisters IS

  constant prescalerBaseAddr: natural := baseAddress;
  constant speedBaseAddr    : natural := baseAddress + 2;
  constant registerLength: positive := 2*busDataIn'length;
  signal prescaler_int: unsigned(registerLength-1 downto 0);
  signal speed_int    :   signed(registerLength-1 downto 0);
  signal enDelayed: std_ulogic;

BEGIN
  ------------------------------------------------------------------------------
                                                         -- store bytes from I2C
  storeBytes: process(reset, clock)
  begin
    if reset = '1' then
      prescaler_int <= (others => '0');
      speed_int <= (others => '0');
      hwOrientation <= (others => '0');
    elsif rising_edge(clock) then
      if en = '1' then
        if write = '1' then
          case to_integer(busAddress) is
            when prescalerBaseAddr =>
              prescaler_int(busDataIn'range) <= unsigned(busDataIn);
            when prescalerBaseAddr+1 =>
              prescaler_int(
                prescaler_int'high downto prescaler_int'length-busDataIn'length
              ) <= unsigned(busDataIn);
            when speedBaseAddr =>
              speed_int(busDataIn'range) <= signed(busDataIn);
            when speedBaseAddr+1 =>
              speed_int(
                speed_int'high downto speed_int'length-busDataIn'length
              ) <= signed(busDataIn);
            when orientationBaseAddress =>
              hwOrientation <= busDataIn(hwOrientation'range);
            when others => null;
          end case;
        end if;
      end if;
    end if;
  end process storeBytes;

  ------------------------------------------------------------------------------
                                                                 -- update words
  storeValues: process(reset, clock)
  begin
    if reset = '1' then
      enDelayed <= '0';
      prescaler <= (others => '0');
      speed <= (others => '0');
    elsif rising_edge(clock) then
      enDelayed <= en;
      if enDelayed = '1' then
        if busAddress(0) = '1' then
          prescaler <= resize(prescaler_int, prescaler'length);
          speed <= resize(speed_int, speed'length);
        end if;
      end if;
    end if;
  end process storeValues;

  --============================================================================
                                                            -- send bytes to I2C
  busDataOut <= (others => '1');

END ARCHITECTURE RTL;




--
-- VHDL Architecture DcMotor.dcMotorController.struct
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 15:35:17 03.09.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

LIBRARY DcMotor;
LIBRARY I2C;

ARCHITECTURE struct OF dcMotorController IS

   -- Architecture declarations
   constant dataBitNb: positive := i2cBitNb-2;
   constant addressBitNb: positive := dataBitNb;

   -- Internal signal declarations
   SIGNAL addr          : unsigned(addressBitNb-1 DOWNTO 0);
   SIGNAL chipAddr      : unsigned(dataBitNb-1-1 DOWNTO 0);
   SIGNAL dataIn        : std_ulogic_vector(dataBitNb-1 DOWNTO 0);
   SIGNAL dataOut       : std_ulogic_vector(dataBitNb-1 DOWNTO 0);
   SIGNAL dataValid     : std_ulogic;
   SIGNAL hwOrientation : std_ulogic_vector(hwOrientationBitNb-1 DOWNTO 0);
   SIGNAL isSelected    : std_ulogic;
   SIGNAL prescaler     : unsigned(prescalerBitNb-1 DOWNTO 0);
   SIGNAL pwmEn         : std_ulogic;
   SIGNAL speed         : signed(speedBitNb-1 DOWNTO 0);
   SIGNAL writeData     : std_ulogic;


   -- Component Declarations
   COMPONENT dcMotorPrescaler
   GENERIC (
      prescalerBitNb : positive := 16
   );
   PORT (
      clock     : IN     std_ulogic ;
      reset     : IN     std_ulogic ;
      pwmEn     : OUT    std_ulogic ;
      prescaler : IN     unsigned (prescalerBitNb-1 DOWNTO 0)
   );
   END COMPONENT;
   COMPONENT dcMotorPwm
   GENERIC (
      speedBitNb : positive := 8
   );
   PORT (
      clock           : IN     std_ulogic ;
      reset           : IN     std_ulogic ;
      en              : IN     std_ulogic ;
      speed           : IN     signed (5-1 DOWNTO 0); -- (speedBitNb-1:0)
      pwmOut          : OUT    std_ulogic ;
      forwards        : OUT    std_ulogic ;
      normalDirection : IN     std_ulogic ;
      testOut         : OUT    std_ulogic_vector (7 DOWNTO 0)
   );
   END COMPONENT;
   COMPONENT dcMotorRegisters
   GENERIC (
      busAddressBitNb        : positive := 8;
      busDataBitNb           : positive := 8;
      speedBitNb             : positive := 8;
      prescalerBitNb         : positive := 16;
      baseAddress            : natural  := 0;
      hwOrientationBitNb     : positive := 3;
      orientationBaseAddress : natural  := 8
   );
   PORT (
      clock         : IN     std_ulogic ;
      reset         : IN     std_ulogic ;
      busDataIn     : IN     std_ulogic_vector (busDataBitNb-1 DOWNTO 0);
      en            : IN     std_ulogic ;
      speed         : OUT    signed (speedBitNb-1 DOWNTO 0);
      busAddress    : IN     unsigned (busAddressBitNb-1 DOWNTO 0);
      prescaler     : OUT    unsigned (prescalerBitNb-1 DOWNTO 0);
      write         : IN     std_ulogic ;
      busDataOut    : OUT    std_ulogic_vector (busDataBitNb-1 DOWNTO 0);
      hwOrientation : OUT    std_ulogic_vector (hwOrientationBitNb-1 DOWNTO 0)
   );
   END COMPONENT;
   COMPONENT i2cTransceiver
   GENERIC (
      dataBitNb : positive := 10
   );
   PORT (
      sDaIn        : IN     std_ulogic ;
      clock        : IN     std_ulogic ;
      reset        : IN     std_ulogic ;
      dataIn       : OUT    std_ulogic_vector (dataBitNb-2-1 DOWNTO 0);
      dataValid    : OUT    std_ulogic ;
      sCl          : IN     std_ulogic ;
      registerAddr : OUT    unsigned (dataBitNb-2-1 DOWNTO 0);
      writeData    : OUT    std_ulogic ;
      sDaOut       : OUT    std_ulogic ;
      dataOut      : IN     std_ulogic_vector (dataBitNb-2-1 DOWNTO 0);
      chipAddr     : OUT    unsigned (dataBitNb-3-1 DOWNTO 0);
      isSelected   : IN     std_ulogic 
   );
   END COMPONENT;

   -- Optional embedded configurations
   -- pragma synthesis_off
   FOR ALL : dcMotorPrescaler USE ENTITY DcMotor.dcMotorPrescaler;
   FOR ALL : dcMotorPwm USE ENTITY DcMotor.dcMotorPwm;
   FOR ALL : dcMotorRegisters USE ENTITY DcMotor.dcMotorRegisters;
   FOR ALL : i2cTransceiver USE ENTITY I2C.i2cTransceiver;
   -- pragma synthesis_on


BEGIN
   -- Architecture concurrent statements
   -- HDL Embedded Text Block 1 eb1
   testOut <= (
     1 => pwmEn,
     2 => hwOrientation(0),
     3 => writeData,
     5 => speed(3),
     6 => speed(2),
     7 => speed(1),
     8 => speed(0),
     others => '0'
   );
   
   

   -- HDL Embedded Text Block 2 eb2
   isSelected <='1' when chipAddr = kartBaseAddress/2 else '0';
   
   


   -- Instance port mappings.
   I_div : dcMotorPrescaler
      GENERIC MAP (
         prescalerBitNb => prescalerBitNb
      )
      PORT MAP (
         clock     => clock,
         reset     => reset,
         pwmEn     => pwmEn,
         prescaler => prescaler
      );
   I_pwm : dcMotorPwm
      GENERIC MAP (
         speedBitNb => speedBitNb
      )
      PORT MAP (
         clock           => clock,
         reset           => reset,
         en              => pwmEn,
         speed           => speed,
         pwmOut          => pwm,
         forwards        => forwards,
         normalDirection => hwOrientation(0),
         testOut         => OPEN
      );
   I_regs : dcMotorRegisters
      GENERIC MAP (
         busAddressBitNb        => addressBitNb,
         busDataBitNb           => dataBitNb,
         speedBitNb             => speedBitNb,
         prescalerBitNb         => prescalerBitNb,
         baseAddress            => dcBaseAddress,
         hwOrientationBitNb     => hwOrientationBitNb,
         orientationBaseAddress => orientationBaseAddress
      )
      PORT MAP (
         clock         => clock,
         reset         => reset,
         busDataIn     => dataIn,
         en            => dataValid,
         speed         => speed,
         busAddress    => addr,
         prescaler     => prescaler,
         write         => writeData,
         busDataOut    => dataOut,
         hwOrientation => hwOrientation
      );
   I_i2c : i2cTransceiver
      GENERIC MAP (
         dataBitNb => i2cBitNb
      )
      PORT MAP (
         sDaIn        => sDaIn,
         clock        => clock,
         reset        => reset,
         dataIn       => dataIn,
         dataValid    => dataValid,
         sCl          => sClIn,
         registerAddr => addr,
         writeData    => writeData,
         sDaOut       => sDaOut,
         dataOut      => dataOut,
         chipAddr     => chipAddr,
         isSelected   => isSelected
      );

END struct;




LIBRARY ieee;
USE ieee.std_logic_1164.all;

PACKAGE gates IS

--  constant gateDelay: time := 1 ns;
  constant gateDelay: time := 0.1 ns;

END gates;




-- VHDL Entity sequential.DFF.symbol
--
-- Created:
--          by - cof.UNKNOWN (WE3479)
--          at - 10:57:17 23.08.2012
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
LIBRARY gates;
  USE gates.gates.all;

ENTITY DFF IS
  GENERIC( 
    delay : time := 1 ns
  );
  PORT( 
    CLK : IN     std_uLogic;
    CLR : IN     std_uLogic;
    D   : IN     std_uLogic;
    Q   : OUT    std_uLogic
  );

-- Declarations

END DFF ;





ARCHITECTURE sim OF DFF IS
BEGIN

  process(clk, clr)
  begin
    if clr = '1' then
      q <= '0' after delay;
    elsif rising_edge(clk) then
      q <= d after delay;
    end if;
  end process;

END sim;




-- VHDL Entity gates.inverter.symbol
--
-- Created:
--          by - cof.UNKNOWN (WE3479)
--          at - 10:55:55 23.08.2012
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
LIBRARY gates;
  USE gates.gates.all;

ENTITY inverter IS
  GENERIC( 
    delay : time := 1 ns
  );
  PORT( 
    in1  : IN     std_uLogic;
    out1 : OUT    std_uLogic
  );

-- Declarations

END inverter ;





ARCHITECTURE sim OF inverter IS
BEGIN
  out1 <= NOT in1 after delay;
END sim;




-- VHDL Entity gates.logic1.symbol
--
-- Created:
--          by - cof.UNKNOWN (WE3479)
--          at - 10:55:54 23.08.2012
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
LIBRARY gates;
  USE gates.gates.all;

ENTITY logic1 IS
  PORT( 
    logic_1 : OUT    std_uLogic
  );

-- Declarations

END logic1 ;





ARCHITECTURE sim OF logic1 IS
BEGIN
  logic_1 <= '1';
END sim;




-- VHDL Entity gates.bufferUlogic.symbol
--
-- Created:
--          by - Everyone.Everyone (WEA20302)
--          at - 16:18:06 08/20/09
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2012.2 (Build 11)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
LIBRARY gates;
  USE gates.gates.all;

ENTITY bufferUlogic IS
   GENERIC( 
      delay : time := gateDelay
   );
   PORT( 
      in1  : IN     std_uLogic;
      out1 : OUT    std_uLogic
   );

-- Declarations

END bufferUlogic ;





ARCHITECTURE sim OF bufferUlogic IS
BEGIN
  out1 <= in1 after delay;
END ARCHITECTURE sim;





--
-- VHDL Architecture Board.FPGA_dcMotorController.struct
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 15:35:18 03.09.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
   USE ieee.std_logic_1164.all;
   USE ieee.numeric_std.all;

LIBRARY gates;
   USE gates.gates.all;

LIBRARY DcMotor;
LIBRARY gates;
LIBRARY sequential;

ARCHITECTURE struct OF FPGA_dcMotorController IS

   -- Architecture declarations
   constant clockFrequency : real := 10.0E6;
   
   constant i2cBitNb : positive := 10;
   
   constant kartBaseAddress: positive := 16#26#; -- must be even
   constant dcBaseAddress: natural := 0;
   constant prescalerBitNb: positive := 16;
   constant speedBitNb: positive := 5;
   
   constant orientationBaseAddress: natural := 10;
   constant hwOrientationBitNb : positive := 3;
   constant testOutBitNb : positive := 8;

   -- Internal signal declarations
   SIGNAL Q            : std_uLogic_vector(1 TO 2);
   SIGNAL forwards     : std_ulogic;
   SIGNAL logic_1      : std_uLogic;
   SIGNAL pwm          : std_ulogic;
   SIGNAL reset        : std_uLogic;
   SIGNAL resetSynch   : std_ulogic;
   SIGNAL resetSynch_n : std_ulogic;
   SIGNAL sClSynch     : std_uLogic;
   SIGNAL sDaIn        : std_uLogic;
   SIGNAL sDaInSynch   : std_uLogic;
   SIGNAL sDaOut       : std_uLogic;
   SIGNAL testOut      : std_ulogic_vector(1 TO testOutBitNb);


   -- Component Declarations
   COMPONENT dcMotorController
   GENERIC (
      i2cBitNb               : positive := 10;
      speedBitNb             : positive := 4;
      prescalerBitNb         : positive := 16;
      kartBaseAddress        : positive := 16#51#;
      dcBaseAddress          : natural  := 0;
      orientationBaseAddress : natural  := 8;
      hwOrientationBitNb     : positive := 3;
      testOutBitNb           : positive := 8
   );
   PORT (
      clock    : IN     std_ulogic ;
      reset    : IN     std_ulogic ;
      sClIn    : IN     std_uLogic ;
      sDaIn    : IN     std_uLogic ;
      forwards : OUT    std_ulogic ;
      pwm      : OUT    std_ulogic ;
      sDaOut   : OUT    std_uLogic ;
      testOut  : OUT    std_ulogic_vector (1 TO testOutBitNb)
   );
   END COMPONENT;
   COMPONENT bufferUlogic
   GENERIC (
      delay : time := gateDelay
   );
   PORT (
      in1  : IN     std_uLogic ;
      out1 : OUT    std_uLogic 
   );
   END COMPONENT;
   COMPONENT inverter
   GENERIC (
      delay : time := 1 ns
   );
   PORT (
      in1  : IN     std_uLogic ;
      out1 : OUT    std_uLogic 
   );
   END COMPONENT;
   COMPONENT logic1
   PORT (
      logic_1 : OUT    std_uLogic 
   );
   END COMPONENT;
   COMPONENT DFF
   GENERIC (
      delay : time := 1 ns
   );
   PORT (
      CLK : IN     std_uLogic ;
      CLR : IN     std_uLogic ;
      D   : IN     std_uLogic ;
      Q   : OUT    std_uLogic 
   );
   END COMPONENT;

   -- Optional embedded configurations
   -- pragma synthesis_off
   FOR ALL : DFF USE ENTITY sequential.DFF;
   FOR ALL : bufferUlogic USE ENTITY gates.bufferUlogic;
   FOR ALL : dcMotorController USE ENTITY DcMotor.dcMotorController;
   FOR ALL : inverter USE ENTITY gates.inverter;
   FOR ALL : logic1 USE ENTITY gates.logic1;
   -- pragma synthesis_on


BEGIN
   -- Architecture concurrent statements
   -- HDL Embedded Text Block 2 eb2
   jtagIo(1) <= '0';
   jtagIo(2) <= '0';
   jtagIo(3) <= '0';
   

   -- HDL Embedded Text Block 3 eb3
   sDa <= '0' when sDaOut = '0'
     else 'Z';
   sDaIn <= sDa;
   

   -- HDL Embedded Text Block 4 eb4
   I_O(1)  <= testOut(3);  -- PWM enable
   I_O(2)  <= '0';
   I_O(3)  <= '0';
   I_O(4)  <= '0';
   I_O(5)  <= '0';
   I_O(6)  <= '0';
   I_O(7)  <= '0';
   I_O(8)  <= '0';
   I_O(9)  <= '0';
   I_O(10) <= '0';
   I_O(11) <= '0';
   I_O(12) <= '0';
   I_O(13) <= '0';
   I_O(14) <= '0';
   I_O(15) <= 'Z';         -- PWM
   I_O(16) <= 'Z';         -- direction
   I_O(17) <= '0';
   I_O(18) <= '0';
   I_O(19) <= '0';
   I_O(20) <= '0';


   -- Instance port mappings.
   I0 : dcMotorController
      GENERIC MAP (
         i2cBitNb               => i2cBitNb,
         speedBitNb             => speedBitNb,
         prescalerBitNb         => prescalerBitNb,
         kartBaseAddress        => kartBaseAddress,
         dcBaseAddress          => dcBaseAddress,
         orientationBaseAddress => orientationBaseAddress,
         hwOrientationBitNb     => hwOrientationBitNb,
         testOutBitNb           => testOutBitNb
      )
      PORT MAP (
         clock    => clock,
         reset    => resetSynch,
         sClIn    => sClSynch,
         sDaIn    => sDaInSynch,
         forwards => forwards,
         pwm      => pwm,
         sDaOut   => sDaOut,
         testOut  => testOut
      );
   I4 : bufferUlogic
      GENERIC MAP (
         delay => gateDelay
      )
      PORT MAP (
         in1  => forwards,
         out1 => I_O(15)
      );
   I5 : bufferUlogic
      GENERIC MAP (
         delay => gateDelay
      )
      PORT MAP (
         in1  => pwm,
         out1 => I_O(16)
      );
   I26 : inverter
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         in1  => resetSynch_n,
         out1 => resetSynch
      );
   I27 : inverter
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         in1  => reset_n,
         out1 => reset
      );
   I3 : logic1
      PORT MAP (
         logic_1 => logic_1
      );
   I21 : DFF
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         CLK => clock,
         CLR => reset,
         D   => logic_1,
         Q   => resetSynch_n
      );
   I23 : DFF
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         CLK => clock,
         CLR => resetSynch,
         D   => sCl,
         Q   => sClSynch
      );
   I24 : DFF
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         CLK => clock,
         CLR => resetSynch,
         D   => sDaIn,
         Q   => sDaInSynch
      );

END struct;




