-- VHDL Entity Board.FPGA_dcMotorController.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 10:47:51 09.08.2013
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
   USE ieee.std_logic_1164.all;
   USE ieee.numeric_std.all;

ENTITY FPGA_dcMotorController IS
   GENERIC( 
      ioBitNb   : positive := 13;
      jtagBitNb : positive := 3
   );
   PORT( 
      clock   : IN     std_ulogic;
      reset_n : IN     std_uLogic;
      sCl     : IN     std_logic;
      jtagIo  : OUT    std_ulogic_vector (1 TO jtagBitNb);
      I_O     : INOUT  std_logic_vector (1 TO ioBitNb);
      sDa     : INOUT  std_logic
   );

-- Declarations

END FPGA_dcMotorController ;





-- VHDL Entity DcMotor.dcMotorController.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 10:46:57 09.08.2013
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY dcMotorController IS
   GENERIC( 
      i2cBitNb               : positive := 10;
      speedBitNb             : positive := 4;
      prescalerBitNb         : positive := 16;
      baseAddress            : natural  := 0;
      hwOrientationBitNb     : positive := 3;
      orientationBaseAddress : natural  := 8;
      testOutBitNb           : positive := 8
   );
   PORT( 
      clock    : IN     std_ulogic;
      reset    : IN     std_ulogic;
      sClIn    : IN     std_uLogic;
      sDaIn    : IN     std_uLogic;
      forwards : OUT    std_ulogic;
      pwm      : OUT    std_ulogic;
      sDaOut   : OUT    std_uLogic;
      testOut  : OUT    std_ulogic_vector (1 TO testOutBitNb)
   );

-- Declarations

END dcMotorController ;





-- VHDL Entity DcMotor.dcMotorPrescaler.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 10:46:57 09.08.2013
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY dcMotorPrescaler IS
   GENERIC( 
      prescalerBitNb : positive := 16
   );
   PORT( 
      clock     : IN     std_ulogic;
      reset     : IN     std_ulogic;
      pwmEn     : OUT    std_ulogic;
      prescaler : IN     unsigned (prescalerBitNb-1 DOWNTO 0)
   );

-- Declarations

END dcMotorPrescaler ;





ARCHITECTURE RTL OF dcMotorPrescaler IS

  signal prescalerCounter: unsigned(prescaler'range);
  signal prescalerDone: std_ulogic;

BEGIN

  divideClock: process(reset, clock)
  begin
    if reset = '1' then
      prescalerCounter <= (others => '0');
    elsif rising_edge(clock) then
      if prescalerDone = '1' then
        prescalerCounter <= (others => '0');
      else
        prescalerCounter <= prescalerCounter + 1;
      end if;
    end if;
  end process divideClock;

  prescalerDone <= '1' when prescalerCounter+1 >= prescaler
    else '0';
  pwmEn <= prescalerDone;

END ARCHITECTURE RTL;




-- VHDL Entity I2C.i2cTransceiver.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 10:09:34 09.08.2013
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY i2cTransceiver IS
   GENERIC( 
      dataBitNb : positive := 8
   );
   PORT( 
      sDaIn     : IN     std_ulogic;
      clock     : IN     std_ulogic;
      reset     : IN     std_ulogic;
      dataIn    : OUT    std_ulogic_vector (dataBitNb-2-1 DOWNTO 0);
      dataValid : OUT    std_ulogic;
      sCl       : IN     std_ulogic;
      addr      : OUT    unsigned (dataBitNb-3-1 DOWNTO 0);
      writeData : OUT    std_ulogic;
      sDaOut    : OUT    std_ulogic;
      dataOut   : IN     std_ulogic_vector (dataBitNb-2-1 DOWNTO 0)
   );

-- Declarations

END i2cTransceiver ;





PACKAGE CommonLib IS

  function requiredBitNb (val : integer) return integer;

END CommonLib;




PACKAGE BODY CommonLib IS

  function requiredBitNb (val : integer) return integer is
    variable powerOfTwo, bitNb : integer;
  begin
    powerOfTwo := 1;
    bitNb := 0;
    while powerOfTwo <= val loop
      powerOfTwo := 2 * powerOfTwo;
      bitNb := bitNb + 1;
    end loop;
    return bitNb;
  end requiredBitNb;

END CommonLib;




-- library Common;
  use work.CommonLib.all;

architecture RTL of i2cTransceiver is
                                                                     -- receiver
  signal clDelayed      : std_uLogic;
  signal daDelayed      : std_uLogic;
  signal clRising       : std_uLogic;
  signal clFalling      : std_uLogic;
  signal daRising       : std_uLogic;
  signal daFalling      : std_uLogic;
  signal startCondition : std_uLogic;
  signal stopCondition  : std_uLogic;
  signal bitCounter     : unsigned(requiredBitNb(dataBitNb)-1 downto 0);
  signal inputShiftReg  : std_ulogic_vector(dataBitNb-2 downto 0);
  signal clRisingDelayed: std_uLogic;
  signal endOfWord      : std_uLogic;
  signal i2cDataValid   : std_uLogic;
  signal i2cData        : std_ulogic_vector(inputShiftReg'high+1 downto 0);
                                                                      -- decoder
  type   rxStateType    is (idle, readAddress, incAddress);
  signal rxState        : rxStateType;
  signal rxAddress      : unsigned(addr'range);
  signal writeMode      : std_uLogic;

  signal loadOutShiftReg: std_uLogic;
  signal enSdaOut       : std_uLogic;
  signal outputShiftReg : std_ulogic_vector(dataBitNb-2-1 downto 0);

begin
  --============================================================================
                                             -- start, stop and other conditions
  delayInputs: process(reset, clock)
  begin
    if reset = '1' then
      clDelayed <= '1';
      daDelayed <= '1';
      clRisingDelayed <= '0';
    elsif rising_edge(clock) then
      clDelayed <= sCl;
      daDelayed <= sDaIn;
      clRisingDelayed <= clRising;
    end if;
  end process delayInputs;

  clRising <= '1' when (sCl = '1') and (clDelayed = '0') else '0';
  clFalling <= '1' when (sCl = '0') and (clDelayed = '1') else '0';
  daRising <= '1' when (sDaIn = '1') and (daDelayed = '0') else '0';
  daFalling <= '1' when (sDaIn = '0') and (daDelayed = '1') else '0';
  startCondition <= '1' when (daFalling = '1') and (sCl = '1') else '0';
  stopCondition <= '1' when (daRising = '1') and (sCl = '1') else '0';

  ------------------------------------------------------------------------------
                                                                  -- bit counter
  countBitNb: process(reset, clock)
  begin
    if reset = '1' then
      bitCounter <= (others => '0');
    elsif rising_edge(clock) then
      if startCondition = '1' then
        bitCounter <= (others => '0');
      elsif stopCondition = '1' then
        bitCounter <= (others => '0');
      elsif clRising = '1' then
        if bitCounter <= dataBitNb then
          bitCounter <= bitCounter + 1;
        else
          bitCounter <= (others => '0');
        end if;
      elsif endOfWord = '1' then
        bitCounter <= (others => '0');
      end if;
    end if;
  end process countBitNb;

  endOfWord <= '1' when (clRisingDelayed = '1') and (bitCounter = dataBitNb-1)
    else '0';

  ------------------------------------------------------------------------------
                                                         -- input shift register
  shiftInputBits: process(reset, clock)
  begin
    if reset = '1' then
      inputShiftReg <= (others => '0');
    elsif rising_edge(clock) then
      if clRising = '1' then
        inputShiftReg(inputShiftReg'high downto 1) <= inputShiftReg(inputShiftReg'high-1 downto 0);
        inputShiftReg(0) <= sDaIn;
      end if;
    end if;
  end process shiftInputBits;

  ------------------------------------------------------------------------------
               -- generate data and control
               -- start and stop: MSB is '1', LSB specifies start/stop
               -- data words: MSB is '0', 8 next bytes are data word, LSB is ack
               -- could be made combinatorial, but sequential is easier to debug
  getWord: process(reset, clock)
  begin
    if reset = '1' then
      i2cDataValid <= '0';
      i2cData <= (others => '0');
    elsif rising_edge(clock) then
      i2cDataValid <= '0';
      if startCondition = '1' then
        i2cDataValid <= '1';
        i2cData <= (others => '0');
        i2cData(i2cData'high) <= '1';
      elsif stopCondition = '1' then
        i2cDataValid <= '1';
        i2cData <= (others => '0');
        i2cData(i2cData'high) <= '1';
        i2cData(0) <= '1';
      elsif endOfWord = '1' then
        i2cDataValid <= '1';
        i2cData <= '0' & inputShiftReg;
      end if;
    end if;
  end process getWord;

  --============================================================================
                                                  -- address and data extraction
  rxFsm: process(reset, clock)
  begin
    if reset = '1' then
      rxState <= idle;
    elsif rising_edge(clock) then
      if stopCondition = '1' then
        rxState <= idle;
      elsif i2cDataValid = '1' then
        case rxState is
          when idle =>
                                         -- to readAddress after start condition
            if (i2cData(i2cData'high) = '1') and (i2cData(0) = '0') then
              rxState <= readAddress;
            end if;
          when readAddress =>
                                              -- to incAddress after readAddress
            rxState <= incAddress;
          when incAddress =>
            rxState <= incAddress;
--            if (i2cData(i2cData'high) = '1') and (i2cData(0) = '1') then
--              rxState <= idle;
--            end if;
        end case;
      end if;
    end if;
  end process rxFsm;

  dataValid <= i2cDataValid and not(i2cData(i2cData'high))
    when rxState = incAddress
    else '0';
  dataIn <= i2cData(dataIn'high+1 downto 1);

  ------------------------------------------------------------------------------
                                                  -- provide running i2c address
  updateAddress: process(reset, clock)
  begin
    if reset = '1' then
      rxAddress <= (others => '0');
      writeMode <= '0';
    elsif rising_edge(clock) then
      if i2cDataValid = '1' then
        case rxState is
          when readAddress =>
            rxAddress <= resize(shift_right(unsigned(i2cData), 2), rxAddress'length);
            writeMode <= not i2cData(1); -- i2cData(1)=0 -> master write to slave
          when incAddress =>
            rxAddress <= rxAddress + 1;
          when others => null;
        end case;
      end if;
    end if;
  end process updateAddress;

  addr <= rxAddress;
  writeData <= writeMode;

  --============================================================================
                                                              -- output controls
  controlAnswering: process(reset, clock)
  begin
    if reset = '1' then
      loadOutShiftReg <= '0';
      enSdaOut <= '0';
    elsif rising_edge(clock) then
      if endOfWord = '1' then
        loadOutShiftReg <= '1';
      elsif clRising = '1' then
        loadOutShiftReg <= '0';
      end if;
      if (bitCounter = 0) and (rxState = incAddress) and (writeMode = '0') then
        if sDaIn /= '0' then
          enSdaOut <= '1';
        end if;
      elsif ( (bitCounter = 8) and (clFalling = '1') ) or (rxState = idle) then
        enSdaOut <= '0';
      end if;
    end if;
  end process controlAnswering;

  ------------------------------------------------------------------------------
                                                        -- output shift register
  shiftOutputBits: process(reset, clock)
  begin
    if reset = '1' then
      outputShiftReg <= (others => '0');
    elsif rising_edge(clock) then
      if loadOutShiftReg = '1' then
        outputShiftReg <= (others => '1');
        outputShiftReg(dataOut'range) <= dataOut;
      elsif clFalling = '1' then
        outputShiftReg(outputShiftReg'high downto 1) <= outputShiftReg(outputShiftReg'high-1 downto 0);
        outputShiftReg(0) <= '0';
      end if;
    end if;
  end process shiftOutputBits;

  ------------------------------------------------------------------------------
                                                        -- reply to read request
  sDaOut <= outputShiftReg(outputShiftReg'high) when enSdaOut = '1'
    else '1';

end RTL;




-- VHDL Entity DcMotor.dcMotorPwm.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 10:46:57 09.08.2013
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY dcMotorPwm IS
   GENERIC( 
      speedBitNb : positive := 8
   );
   PORT( 
      clock           : IN     std_ulogic;
      reset           : IN     std_ulogic;
      en              : IN     std_ulogic;
      speed           : IN     signed (speedBitNb-1 DOWNTO 0);
      pwmOut          : OUT    std_ulogic;
      forwards        : OUT    std_ulogic;
      invertDirection : IN     std_ulogic
   );

-- Declarations

END dcMotorPwm ;





ARCHITECTURE RTL OF dcMotorPwm IS

  signal sign: std_ulogic;
  signal speedAmplitude: unsigned(speed'high-1 downto 0);
  signal sawtooth: unsigned(speedAmplitude'range);

BEGIN
  ------------------------------------------------------------------------------
                                                                    -- direction
  sign <= speed(speed'high);
  forwards <= not sign when invertDirection = '0'
    else sign;

  ------------------------------------------------------------------------------
                                                                          -- PWM
  speedAmplitude <= unsigned(speed(speedAmplitude'range)) when sign = '0'
    else unsigned(-speed(speedAmplitude'range));

  storeValues: process(reset, clock)
  begin
    if reset = '1' then
      sawtooth <= (others => '0');
    elsif rising_edge(clock) then
      if en = '1' then
        sawtooth <= sawtooth + 1;
      end if;
    end if;
  end process storeValues;

  pwmOut <= '1' when speedAmplitude >= sawtooth
    else '0';

END ARCHITECTURE RTL;




-- VHDL Entity DcMotor.dcMotorRegisters.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 10:46:57 09.08.2013
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY dcMotorRegisters IS
   GENERIC( 
      busAddressBitNb        : positive := 8;
      busDataBitNb           : positive := 8;
      speedBitNb             : positive := 8;
      prescalerBitNb         : positive := 16;
      baseAddress            : natural  := 0;
      hwOrientationBitNb     : positive := 3;
      orientationBaseAddress : natural  := 8
   );
   PORT( 
      clock         : IN     std_ulogic;
      reset         : IN     std_ulogic;
      busDataIn     : IN     std_ulogic_vector (busDataBitNb-1 DOWNTO 0);
      en            : IN     std_ulogic;
      speed         : OUT    signed (speedBitNb-1 DOWNTO 0);
      busAddress    : IN     unsigned (busAddressBitNb-1 DOWNTO 0);
      prescaler     : OUT    unsigned (prescalerBitNb-1 DOWNTO 0);
      write         : IN     std_ulogic;
      busDataOut    : OUT    std_ulogic_vector (busDataBitNb-1 DOWNTO 0);
      hwOrientation : OUT    std_ulogic_vector (hwOrientationBitNb-1 DOWNTO 0)
   );

-- Declarations

END dcMotorRegisters ;





ARCHITECTURE RTL OF dcMotorRegisters IS

  constant prescalerBaseAddr: natural := baseAddress;
  constant speedBaseAddr    : natural := baseAddress + 2;
  constant registerLength: positive := 2*busDataIn'length;
  signal prescaler_int: unsigned(registerLength-1 downto 0);
  signal speed_int    :   signed(registerLength-1 downto 0);
  signal enDelayed: std_ulogic;

BEGIN
  ------------------------------------------------------------------------------
                                                         -- store bytes from I2C
  storeBytes: process(reset, clock)
  begin
    if reset = '1' then
      prescaler_int <= (others => '0');
      speed_int <= (others => '0');
      hwOrientation <= (others => '0');
    elsif rising_edge(clock) then
      if en = '1' then
        if write = '1' then
          case to_integer(busAddress) is
            when prescalerBaseAddr =>
              prescaler_int(busDataIn'range) <= unsigned(busDataIn);
            when prescalerBaseAddr+1 =>
              prescaler_int(
                prescaler_int'high downto prescaler_int'length-busDataIn'length
              ) <= unsigned(busDataIn);
            when speedBaseAddr =>
              speed_int(busDataIn'range) <= signed(busDataIn);
            when speedBaseAddr+1 =>
              speed_int(
                speed_int'high downto speed_int'length-busDataIn'length
              ) <= signed(busDataIn);
            when orientationBaseAddress =>
              hwOrientation <= busDataIn(hwOrientation'range);
            when others => null;
          end case;
        end if;
      end if;
    end if;
  end process storeBytes;

  ------------------------------------------------------------------------------
                                                                 -- update words
  storeValues: process(reset, clock)
  begin
    if reset = '1' then
      enDelayed <= '0';
      prescaler <= (others => '0');
      speed <= (others => '0');
    elsif rising_edge(clock) then
      enDelayed <= en;
      if enDelayed = '1' then
        if busAddress(0) = '0' then
          prescaler <= resize(prescaler_int, prescaler'length);
          speed <= resize(speed_int, speed'length);
        end if;
      end if;
    end if;
  end process storeValues;

  --============================================================================
                                                            -- send bytes to I2C
  busDataOut <= (others => '1');

END ARCHITECTURE RTL;




--
-- VHDL Architecture DcMotor.dcMotorController.struct
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 10:46:57 09.08.2013
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

-- LIBRARY DcMotor;
-- LIBRARY I2C;

ARCHITECTURE struct OF dcMotorController IS

   -- Architecture declarations
   constant dataBitNb: positive := i2cBitNb-2;
   constant addressBitNb: positive := dataBitNb-1;

   -- Internal signal declarations
   SIGNAL addr          : unsigned(addressBitNb-1 DOWNTO 0);
   SIGNAL dataIn        : std_ulogic_vector(dataBitNb-1 DOWNTO 0);
   SIGNAL dataOut       : std_ulogic_vector(dataBitNb-1 DOWNTO 0);
   SIGNAL dataValid     : std_ulogic;
   SIGNAL hwOrientation : std_ulogic_vector(hwOrientationBitNb-1 DOWNTO 0);
   SIGNAL prescaler     : unsigned(prescalerBitNb-1 DOWNTO 0);
   SIGNAL pwmEn         : std_ulogic;
   SIGNAL speed         : signed(speedBitNb-1 DOWNTO 0);
   SIGNAL writeData     : std_ulogic;


   -- Component Declarations
   COMPONENT dcMotorPrescaler
   GENERIC (
      prescalerBitNb : positive := 16
   );
   PORT (
      clock     : IN     std_ulogic ;
      reset     : IN     std_ulogic ;
      pwmEn     : OUT    std_ulogic ;
      prescaler : IN     unsigned (prescalerBitNb-1 DOWNTO 0)
   );
   END COMPONENT;
   COMPONENT dcMotorPwm
   GENERIC (
      speedBitNb : positive := 8
   );
   PORT (
      clock           : IN     std_ulogic ;
      reset           : IN     std_ulogic ;
      en              : IN     std_ulogic ;
      speed           : IN     signed (speedBitNb-1 DOWNTO 0);
      pwmOut          : OUT    std_ulogic ;
      forwards        : OUT    std_ulogic ;
      invertDirection : IN     std_ulogic 
   );
   END COMPONENT;
   COMPONENT dcMotorRegisters
   GENERIC (
      busAddressBitNb        : positive := 8;
      busDataBitNb           : positive := 8;
      speedBitNb             : positive := 8;
      prescalerBitNb         : positive := 16;
      baseAddress            : natural  := 0;
      hwOrientationBitNb     : positive := 3;
      orientationBaseAddress : natural  := 8
   );
   PORT (
      clock         : IN     std_ulogic ;
      reset         : IN     std_ulogic ;
      busDataIn     : IN     std_ulogic_vector (busDataBitNb-1 DOWNTO 0);
      en            : IN     std_ulogic ;
      speed         : OUT    signed (speedBitNb-1 DOWNTO 0);
      busAddress    : IN     unsigned (busAddressBitNb-1 DOWNTO 0);
      prescaler     : OUT    unsigned (prescalerBitNb-1 DOWNTO 0);
      write         : IN     std_ulogic ;
      busDataOut    : OUT    std_ulogic_vector (busDataBitNb-1 DOWNTO 0);
      hwOrientation : OUT    std_ulogic_vector (hwOrientationBitNb-1 DOWNTO 0)
   );
   END COMPONENT;
   COMPONENT i2cTransceiver
   GENERIC (
      dataBitNb : positive := 8
   );
   PORT (
      sDaIn     : IN     std_ulogic ;
      clock     : IN     std_ulogic ;
      reset     : IN     std_ulogic ;
      dataIn    : OUT    std_ulogic_vector (dataBitNb-2-1 DOWNTO 0);
      dataValid : OUT    std_ulogic ;
      sCl       : IN     std_ulogic ;
      addr      : OUT    unsigned (dataBitNb-3-1 DOWNTO 0);
      writeData : OUT    std_ulogic ;
      sDaOut    : OUT    std_ulogic ;
      dataOut   : IN     std_ulogic_vector (dataBitNb-2-1 DOWNTO 0)
   );
   END COMPONENT;

   -- Optional embedded configurations
   -- pragma synthesis_off
--    FOR ALL : dcMotorPrescaler USE ENTITY DcMotor.dcMotorPrescaler;
--    FOR ALL : dcMotorPwm USE ENTITY DcMotor.dcMotorPwm;
--    FOR ALL : dcMotorRegisters USE ENTITY DcMotor.dcMotorRegisters;
--    FOR ALL : i2cTransceiver USE ENTITY I2C.i2cTransceiver;
   -- pragma synthesis_on


BEGIN
   -- Architecture concurrent statements
   -- HDL Embedded Text Block 1 eb1
   testOut <= (
     1 => pwmEn,
     2 => hwOrientation(0),
     3 => writeData,
     5 => speed(3),
     6 => speed(2),
     7 => speed(1),
     8 => speed(0),
     others => '0'
   );
   
   


   -- Instance port mappings.
   I_div : dcMotorPrescaler
      GENERIC MAP (
         prescalerBitNb => prescalerBitNb
      )
      PORT MAP (
         clock     => clock,
         reset     => reset,
         pwmEn     => pwmEn,
         prescaler => prescaler
      );
   I_pwm : dcMotorPwm
      GENERIC MAP (
         speedBitNb => speedBitNb
      )
      PORT MAP (
         clock           => clock,
         reset           => reset,
         en              => pwmEn,
         speed           => speed,
         pwmOut          => pwm,
         forwards        => forwards,
         invertDirection => hwOrientation(0)
      );
   I_regs : dcMotorRegisters
      GENERIC MAP (
         busAddressBitNb        => addressBitNb,
         busDataBitNb           => dataBitNb,
         speedBitNb             => speedBitNb,
         prescalerBitNb         => prescalerBitNb,
         baseAddress            => baseAddress,
         hwOrientationBitNb     => hwOrientationBitNb,
         orientationBaseAddress => orientationBaseAddress
      )
      PORT MAP (
         clock         => clock,
         reset         => reset,
         busDataIn     => dataIn,
         en            => dataValid,
         speed         => speed,
         busAddress    => addr,
         prescaler     => prescaler,
         write         => writeData,
         busDataOut    => dataOut,
         hwOrientation => hwOrientation
      );
   I_i2c : i2cTransceiver
      GENERIC MAP (
         dataBitNb => i2cBitNb
      )
      PORT MAP (
         sDaIn     => sDaIn,
         clock     => clock,
         reset     => reset,
         dataIn    => dataIn,
         dataValid => dataValid,
         sCl       => sClIn,
         addr      => addr,
         writeData => writeData,
         sDaOut    => sDaOut,
         dataOut   => dataOut
      );

END struct;




LIBRARY ieee;
USE ieee.std_logic_1164.all;

PACKAGE gates IS

  constant gateDelay: time := 1 ns;
--  constant gateDelay: time := 0.1 ns;

END gates;




-- VHDL Entity sequential.DFF.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 13:30:15 29.07.2013
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
-- LIBRARY gates;
  use work.gates.all;

ENTITY DFF IS
   GENERIC( 
      delay : time := 1 ns
   );
   PORT( 
      CLK : IN     std_uLogic;
      CLR : IN     std_uLogic;
      D   : IN     std_uLogic;
      Q   : OUT    std_uLogic
   );

-- Declarations

END DFF ;





ARCHITECTURE sim OF DFF IS
BEGIN

  process(clk, clr)
  begin
    if clr = '1' then
      q <= '0' after delay;
    elsif rising_edge(clk) then
      q <= d after delay;
    end if;
  end process;

END sim;




-- VHDL Entity gates.inverter.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 13:30:15 29.07.2013
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
-- LIBRARY gates;
  use work.gates.all;

ENTITY inverter IS
   GENERIC( 
      delay : time := 1 ns
   );
   PORT( 
      in1  : IN     std_uLogic;
      out1 : OUT    std_uLogic
   );

-- Declarations

END inverter ;





ARCHITECTURE sim OF inverter IS
BEGIN
  out1 <= NOT in1 after delay;
END sim;




-- VHDL Entity gates.logic1.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 13:30:15 29.07.2013
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
-- LIBRARY gates;
  use work.gates.all;

ENTITY logic1 IS
   PORT( 
      logic_1 : OUT    std_uLogic
   );

-- Declarations

END logic1 ;





ARCHITECTURE sim OF logic1 IS
BEGIN
  logic_1 <= '1';
END sim;




-- VHDL Entity gates.bufferUlogic.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 13:30:15 29.07.2013
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
-- LIBRARY gates;
  use work.gates.all;

ENTITY bufferUlogic IS
   GENERIC( 
      delay : time := gateDelay
   );
   PORT( 
      in1  : IN     std_uLogic;
      out1 : OUT    std_uLogic
   );

-- Declarations

END bufferUlogic ;





ARCHITECTURE sim OF bufferUlogic IS
BEGIN
  out1 <= in1 after delay;
END ARCHITECTURE sim;





--
-- VHDL Architecture Board.FPGA_dcMotorController.struct
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 10:47:51 09.08.2013
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
   USE ieee.std_logic_1164.all;
   USE ieee.numeric_std.all;

-- LIBRARY gates;
   use work.gates.all;

-- LIBRARY DcMotor;
-- LIBRARY gates;
-- LIBRARY sequential;

ARCHITECTURE struct OF FPGA_dcMotorController IS

   -- Architecture declarations
   constant clockFrequency : real := 10.0E6;
   
   constant i2cBitNb : positive := 10;
   
   constant baseAddress: natural := 0;
   constant prescalerBitNb: positive := 16;
   constant speedBitNb: positive := 5;
   
   constant orientationBaseAddress: natural := 10;
   constant hwOrientationBitNb : positive := 3;
   constant testOutBitNb : positive := 8;

   -- Internal signal declarations
   SIGNAL Q            : std_uLogic_vector(1 TO 2);
   SIGNAL forwards     : std_ulogic;
   SIGNAL logic_1      : std_uLogic;
   SIGNAL pwm          : std_ulogic;
   SIGNAL reset        : std_uLogic;
   SIGNAL resetSynch   : std_ulogic;
   SIGNAL resetSynch_n : std_ulogic;
   SIGNAL sClSynch     : std_uLogic;
   SIGNAL sDaIn        : std_uLogic;
   SIGNAL sDaInSynch   : std_uLogic;
   SIGNAL sDaOut       : std_uLogic;
   SIGNAL testOut      : std_ulogic_vector(1 TO testOutBitNb);


   -- Component Declarations
   COMPONENT dcMotorController
   GENERIC (
      i2cBitNb               : positive := 10;
      speedBitNb             : positive := 4;
      prescalerBitNb         : positive := 16;
      baseAddress            : natural  := 0;
      hwOrientationBitNb     : positive := 3;
      orientationBaseAddress : natural  := 8;
      testOutBitNb           : positive := 8
   );
   PORT (
      clock    : IN     std_ulogic ;
      reset    : IN     std_ulogic ;
      sClIn    : IN     std_uLogic ;
      sDaIn    : IN     std_uLogic ;
      forwards : OUT    std_ulogic ;
      pwm      : OUT    std_ulogic ;
      sDaOut   : OUT    std_uLogic ;
      testOut  : OUT    std_ulogic_vector (1 TO testOutBitNb)
   );
   END COMPONENT;
   COMPONENT bufferUlogic
   GENERIC (
      delay : time := gateDelay
   );
   PORT (
      in1  : IN     std_uLogic ;
      out1 : OUT    std_uLogic 
   );
   END COMPONENT;
   COMPONENT inverter
   GENERIC (
      delay : time := 1 ns
   );
   PORT (
      in1  : IN     std_uLogic ;
      out1 : OUT    std_uLogic 
   );
   END COMPONENT;
   COMPONENT logic1
   PORT (
      logic_1 : OUT    std_uLogic 
   );
   END COMPONENT;
   COMPONENT DFF
   GENERIC (
      delay : time := 1 ns
   );
   PORT (
      CLK : IN     std_uLogic ;
      CLR : IN     std_uLogic ;
      D   : IN     std_uLogic ;
      Q   : OUT    std_uLogic 
   );
   END COMPONENT;

   -- Optional embedded configurations
   -- pragma synthesis_off
--    FOR ALL : DFF USE ENTITY sequential.DFF;
--    FOR ALL : bufferUlogic USE ENTITY gates.bufferUlogic;
--    FOR ALL : dcMotorController USE ENTITY DcMotor.dcMotorController;
--    FOR ALL : inverter USE ENTITY gates.inverter;
--    FOR ALL : logic1 USE ENTITY gates.logic1;
   -- pragma synthesis_on


BEGIN
   -- Architecture concurrent statements
   -- HDL Embedded Text Block 2 eb2
   jtagIo(1) <= '0';
   jtagIo(2) <= '0';
   jtagIo(3) <= '0';
   

   -- HDL Embedded Text Block 3 eb3
   sDa <= '0' when sDaOut = '0'
     else 'Z';
   sDaIn <= sDa;
   

   -- HDL Embedded Text Block 4 eb4
   I_O(3)  <= testOut(3);
   I_O(4)  <= '0';
   I_O(5)  <= '0';
   I_O(6)  <= '0';
   I_O(7)  <= '0';
   I_O(8)  <= '0';
   I_O(9)  <= '0';
   I_O(10) <= testOut(8);
   I_O(11) <= testOut(6);
   I_O(12) <= testOut(7);
   I_O(13) <= testOut(5);
   


   -- Instance port mappings.
   I0 : dcMotorController
      GENERIC MAP (
         i2cBitNb               => i2cBitNb,
         speedBitNb             => speedBitNb,
         prescalerBitNb         => prescalerBitNb,
         baseAddress            => baseAddress,
         hwOrientationBitNb     => hwOrientationBitNb,
         orientationBaseAddress => orientationBaseAddress,
         testOutBitNb           => testOutBitNb
      )
      PORT MAP (
         clock    => clock,
         reset    => resetSynch,
         sClIn    => sClSynch,
         sDaIn    => sDaInSynch,
         forwards => forwards,
         pwm      => pwm,
         sDaOut   => sDaOut,
         testOut  => testOut
      );
   I4 : bufferUlogic
      GENERIC MAP (
         delay => gateDelay
      )
      PORT MAP (
         in1  => forwards,
         out1 => I_O(2)
      );
   I5 : bufferUlogic
      GENERIC MAP (
         delay => gateDelay
      )
      PORT MAP (
         in1  => pwm,
         out1 => I_O(1)
      );
   I26 : inverter
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         in1  => resetSynch_n,
         out1 => resetSynch
      );
   I27 : inverter
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         in1  => reset_n,
         out1 => reset
      );
   I3 : logic1
      PORT MAP (
         logic_1 => logic_1
      );
   I21 : DFF
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         CLK => clock,
         CLR => reset,
         D   => logic_1,
         Q   => resetSynch_n
      );
   I23 : DFF
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         CLK => clock,
         CLR => resetSynch,
         D   => sCl,
         Q   => sClSynch
      );
   I24 : DFF
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         CLK => clock,
         CLR => resetSynch,
         D   => sDaIn,
         Q   => sDaInSynch
      );

END struct;




