ARCHITECTURE RTL OF hallCounters IS

  signal pulsesDelayed, pulsesChanged: std_ulogic_vector(1 to sensorNb);
  type counterSetType is array(1 to sensorNb) of unsigned(countBitNb-1 downto 0);
  signal counterSet: counterSetType;

BEGIN
  ------------------------------------------------------------------------------
                                                             -- find transitions
  delayPulses: process(reset, clock)
  begin
    if reset = '1' then
      pulsesDelayed <= (others => '0');
    elsif rising_edge(clock) then
      pulsesDelayed <= hallPulses;
    end if;
  end process delayPulses;

  pulsesChanged <= hallPulses xor pulsesDelayed;
  hallChanged <= pulsesChanged;

  ------------------------------------------------------------------------------
                                                      -- count transition number
  countPulses: process(reset, clock)
  begin
    if reset = '1' then
      counterSet <= (others => (others => '0'));
    elsif rising_edge(clock) then
      for index in pulsesChanged'range loop
        if pulsesChanged(index) = '1' then
          counterSet(index) <= counterSet(index) + 1;
        end if;
      end loop;
    end if;
  end process countPulses;

  ------------------------------------------------------------------------------
                                                        -- pack to single vector
  assignWords: process(counterSet)
    variable offset : natural;
  begin
    for index in counterSet'range loop
      offset := (index-1) * counterSet(index)'length;
      position(counterSet(index)'high+offset downto offset) <= counterSet(index);
    end loop;
  end process assignWords;

END ARCHITECTURE RTL;
