ARCHITECTURE RTL OF ultrasoundRanger IS

  signal usCounter: unsigned(usDivideBitNb-1 downto 0);
  signal usEn: std_ulogic;

  type   rangerStateType is (
    idle, sendPulse
  );
  signal rangerState: rangerStateType;

BEGIN
  ------------------------------------------------------------------------------
                                                 -- generate a 1 us enable pulse
  countTo1us: process(reset, clock)
  begin
    if reset = '1' then
      usCounter <= (others => '0');
    elsif rising_edge(clock) then
      usCounter <= usCounter + 1;
    end if;
  end process countTo1us;

  usEn <= '1' when usCounter+1 = 0
    else '0';

  ------------------------------------------------------------------------------
                                                               -- controller FSM
  rangerFSM: process(reset, clock)
  begin
    if reset = '1' then
      rangerState <= idle;
    elsif rising_edge(clock) then
      case rangerState is
        when idle =>
          if usEn = '1' then
            rangerState <= sendPulse;
          end if;
        when sendPulse =>
          if usEn = '1' then
            rangerState <= idle;
          end if;
        when others => null;
      end case;
    end if;
  end process rangerFSM;

  rangerControls: process(rangerState)
  begin
    startMeasure <= '0';
    case rangerState is
      when sendPulse =>
        startMeasure <= '1';
      when others => null;
    end case;
  end process rangerControls;

END ARCHITECTURE RTL;
