@echo off
SETLOCAL EnableExtensions EnableDelayedExpansion
REM ----------------------------------------------------------------------------
REM Define environment variables
REM
set DESIGN_NAME=sensorsInterface
set HDS_PROJECT_DIR=%CD%

REM set and verify tool installation paths
REM first check path as set on my PC, if not found, check official labo14 path
set HDS_HOME=C:\eda\mentor\HDS2009.2
if not exist !HDS_HOME!\ (
	set HDS_HOME=C:\eda\HDS
	if not exist !HDS_HOME!\ (
		echo ERROR: No valid installtion of HDS found: please verify your HDS_HOME settings.
		pause
		exit
	)
)
set MODELSIM_HOME=C:\eda\modeltech_6.6g
if not exist !MODELSIM_HOME!\ (
	set MODELSIM_HOME=C:\eda\Modelsim
	if not exist !MODELSIM_HOME!\ (
		echo ERROR: No valid installtion of ModelSim found: please verify your MODELSIM_HOME settings.
		pause
		exit
	)
)
set ACTEL_HOME=C:\eda\Microsemi\Libero_v11.0
if not exist !ACTEL_HOME!\ (
	set ACTEL_HOME=C:\eda\Microsemi\Libero_v11.0
	if not exist !ACTEL_HOME!\ (
		echo ERROR: No valid installtion of Libero found: please verify your ACTEL_HOME settings.
		pause
		exit
	)
)
set ISE_HOME=C:\eda\Xilinx\ISE_DS\ISE\bin\nt
if not exist !ISE_HOME!\ (
	set ISE_HOME=C:\eda\Xilinx\12.4\ISE_DS\ISE\bin\nt64
	if not exist !ISE_HOME!\ (
		set ISE_HOME=C:\eda\Xilinx\14.5\ISE_DS\ISE\bin\nt64
		if not exist !ISE_HOME!\ (
			echo ERROR: No valid installtion of ISE found: please verify your ISE_HOME settings.
			pause
			exit
		)
	)
)

REM set existing library installation paths
set HDS_PRIMS_DIR=HDS_PROJECT_DIR%\..\libs
if not exist !HDS_PRIMS_DIR!\ (
	set HDS_PRIMS_DIR=R:/Modules/SI/2131_ELE/ELN/Labs/Libraries
	if not exist !HDS_PRIMS_DIR!\ (
		echo ERROR: No valid libraries found: please verify your HDS_PRIMS_DIR settings.
		pause
		exit
	)
)
set HDS_LIBS_DIR=%HDS_PROJECT_DIR%\..\libs
if not exist !HDS_LIBS_DIR!\ (
	set HDS_LIBS_DIR=R:/Modules/SI/2131_ELE/ELN/Labs/Libraries
	if not exist !HDS_LIBS_DIR!\ (
		echo ERROR: No valid libraries found: please verify your HDS_LIBS_DIR settings.
		pause
		exit
	)
)

set HDS_LIBS=%HDS_PROJECT_DIR%\Prefs\hds.hdp
set HDS_USER_HOME=%HDS_PROJECT_DIR%\Prefs\hds_user
set HDS_TEAM_HOME=%HDS_PROJECT_DIR%\Prefs\hds_team
set HDS_LIB_BOARD=%HDS_PROJECT_DIR%\board
set HDS_CONCAT_DIR=%HDS_LIB_BOARD%\concat

set SIMULATION_DIR=%HDS_PROJECT_DIR:\=/%/Simulation

set ACTEL_WORK_DIR=Board\actel
set ACTEL_IMPL_DIR=designer\impl1
set ACTEL_USER_DIR=%HDS_PROJECT_DIR%\%ACTEL_WORK_DIR%

REM ----------------------------------------------------------------------------
REM Prepare scratch directory
REM
ver | findstr /i "5\.1\." > nul 
IF %ERRORLEVEL% EQU 0 (
    REM Windows XP and older
    set SCRATCH_DIR=%USERPROFILE:C:\Documents and Settings=C:\temp\EDA%\%DESIGN_NAME%
) else (
    REM Windows Vista and newer
    set SCRATCH_DIR=%USERPROFILE:C:\Users=C:\temp\EDA%\%DESIGN_NAME%
)
set ACTEL_SCRATCH_DIR="%SCRATCH_DIR%\%DESIGN_NAME%\%ACTEL_WORK_DIR%"
set DESIGN_SCRATCH_DIR=%SCRATCH_DIR%\%DESIGN_NAME%
echo Copying place and route files to scratch directory: %DESIGN_SCRATCH_DIR%
rmdir /S /Q "%DESIGN_SCRATCH_DIR%"
mkdir "%DESIGN_SCRATCH_DIR%"
xcopy /S /I /Q %HDS_PROJECT_DIR%\%ACTEL_WORK_DIR% "%SCRATCH_DIR%\%DESIGN_NAME%\%ACTEL_WORK_DIR%"
del "%ACTEL_SCRATCH_DIR%\%ACTEL_IMPL_DIR%\FPGA_%DESIGN_NAME%.pdb"

set LIBERO_WORK_DIR=%ACTEL_WORK_DIR%
REM set LIBERO_WORK_DIR=Board\libero
set LIBERO_IMPL_DIR=designer\impl1
set LIBERO_USER_DIR=%HDS_PROJECT_DIR%\%LIBERO_WORK_DIR%
set LIBERO_USER_IMPL="%LIBERO_USER_DIR%\%LIBERO_IMPL_DIR%"
set LIBERO_SCRATCH_DIR=%DESIGN_SCRATCH_DIR%\%LIBERO_WORK_DIR%
set LIBERO_SCRATCH_PRJX=%LIBERO_SCRATCH_DIR%\%DESIGN_NAME%.prjx
set LIBERO_SCRATCH_IMPL="%LIBERO_SCRATCH_DIR%\%LIBERO_IMPL_DIR%"

REM ----------------------------------------------------------------------------
REM Prepare scratch directory
REM
echo.
echo Copying place and route files to scratch directory: %LIBERO_SCRATCH_DIR%
rmdir /S /Q "%DESIGN_SCRATCH_DIR%"
mkdir "%DESIGN_SCRATCH_DIR%"
xcopy /S /I /Q %LIBERO_USER_DIR% "%LIBERO_SCRATCH_DIR%"
del "%LIBERO_SCRATCH_IMPL%\FPGA_%DESIGN_NAME%.pdb"

REM ----------------------------------------------------------------------------
REM Delete intermediate files
REM
echo.
echo Deleting automatically generated files in: %HDS_PROJECT_DIR%
del /s %HDS_PROJECT_DIR%\*.bak %HDS_PROJECT_DIR%\*.lck
del /s %HDS_PROJECT_DIR%\*_entity.vhd %HDS_PROJECT_DIR%\*_struct.vhd %HDS_PROJECT_DIR%\*_fsm.vhd
del /s %HDS_PROJECT_DIR%\*.vhg

REM ----------------------------------------------------------------------------
REM Launch Applications
REM
echo.
echo Launching programs
echo Waiting until programs finished... WARNING: DO NOT CLOSE THIS WINDOW!
%windir%\system32\cmd.exe /c start /wait !HDS_HOME!\bin\hdldesigner.exe
echo Programs finished

REM ----------------------------------------------------------------------------
REM Copy files back from the scratch directory
REM
echo.
echo Copying and overwrite place and route files to user directory
REM pause
copy /Y %LIBERO_SCRATCH_PRJX% %LIBERO_WORK_DIR%\
copy /Y %LIBERO_SCRATCH_IMPL%\FPGA_%DESIGN_NAME%.adb %LIBERO_USER_IMPL%\
copy /Y %LIBERO_SCRATCH_IMPL%\FPGA_%DESIGN_NAME%.pdb %LIBERO_USER_IMPL%\
copy /Y %LIBERO_SCRATCH_IMPL%\FPGA_%DESIGN_NAME%.ide_des %LIBERO_USER_IMPL%\
REM pause
REM rmdir /S /Q "%DESIGN_SCRATCH_DIR%"
ENDLOCAL
