LIBRARY std;
  USE std.textio.ALL;
LIBRARY ieee;
  USE ieee.std_logic_textio.ALL;

ARCHITECTURE test OF stepperMotorController_tester IS

  constant clockPeriod : time := 1.0/clockFrequency * 1 sec;
  signal sClock: std_uLogic := '1';

  constant i2cPeriod : time := 1.0/i2cBaudRate * 1 sec;
  constant i2cRegisterNb: positive := 16;
  type i2cRegistersType is array(0 to i2cRegisterNb-1) of integer;
  signal i2cBaseAddress: natural;
  signal i2cRegisters: i2cRegistersType;
  signal i2cDataLength: positive;
  signal i2cWrite: std_ulogic;
  signal i2cSend: std_ulogic;
  signal i2cDone: std_ulogic;

  signal i2cDataOut: unsigned(i2cBitNb-1-1 downto 0);
  signal i2cSendWord: std_ulogic;
  signal i2cWordDone: std_ulogic;
  signal i2cDataIn: integer;

  constant i2cDataBitNb : positive := i2cBitNb-2;
  constant angleMaxValue: positive := 2**angleBitNb-1;

BEGIN
  ------------------------------------------------------------------------------
                                                              -- reset and clock
  reset <= '1', '0' after 4*clockPeriod;

  sClock <= not sClock after clockPeriod/2;
  clock <= transport sClock after 0.9*clockPeriod;

  ------------------------------------------------------------------------------
                                                                -- test sequence
  process
  begin
    testMode <= '0';
    stepperEnd <= '0';
    i2cSend <= '0';
    i2cWrite <= '0';
    i2cBaseAddress <= 0;
    i2cRegisters <= (others => 0);
    i2cDataLength <= i2cRegisterNb;

    wait for 1 ns;
    write(output,
      lf & lf & lf &
      "----------------------------------------------------------------" & lf &
      "-- Starting testbench" & lf &
      "--" &
      lf & lf
    );
                                                           -- actuate end switch
    wait for 10*i2cPeriod;
    write(output,
      "Activating end of turn switch" &
      " at time " & integer'image(now/1 us) & " us" &
      lf & lf
    );
    stepperEnd <= '1', '0' after i2cPeriod;
                                                   -- send prescaler large value
    wait for 10*i2cPeriod;
    stepperEnd <= '0';
    write(output,
      "Sending prescaler value" &
      " at time " & integer'image(now/1 us) & " us" &
      lf & lf
    );
    i2cBaseAddress <= baseAddress; wait for 0 ns;
    i2cDataLength <= 2;
    i2cRegisters(i2cBaseAddress)   <= 16#34#;
    i2cRegisters(i2cBaseAddress+1) <= 16#12#;
    i2cWrite <= '1';
    I2Csend <= '1', '0' after 1 ns;
    wait until i2cDone = '1';
                                                               -- send prescaler
    wait for 10*i2cPeriod;
    stepperEnd <= '0';
    write(output,
      "Sending prescaler value" &
      " at time " & integer'image(now/1 us) & " us" &
      lf & lf
    );
    i2cBaseAddress <= baseAddress; wait for 0 ns;
    i2cDataLength <= 2;
    i2cRegisters(i2cBaseAddress)   <= 3;
    i2cRegisters(i2cBaseAddress+1) <=  0;
    i2cWrite <= '1';
    I2Csend <= '1', '0' after 1 ns;
    wait until i2cDone = '1';
                                                              -- send half angle
    wait for 10*i2cPeriod;
    write(output,
      "Sending turn control to half angle" &
      " at time " & integer'image(now/1 us) & " us" &
      lf & lf
    );
    i2cBaseAddress <= baseAddress + 2; wait for 0 ns;
    i2cDataLength <= 2;
    i2cRegisters(i2cBaseAddress)   <= (angleMaxValue+1)/2 mod 2**i2cDataBitNb;
    i2cRegisters(i2cBaseAddress+1) <= (angleMaxValue+1)/2 / 2**i2cDataBitNb;
    i2cWrite <= '1';
    i2cSend <= '1', '0' after 1 ns;
                                                               -- ask for status
    wait for 1000*i2cPeriod;
    write(output,
      "Asking for status" &
      " at time " & integer'image(now/1 us) & " us" &
      lf &
      "  Should be FFFDh (not reached)" &
      lf & lf
    );
    i2cBaseAddress <= baseAddress; wait for 0 ns;
    i2cDataLength <= 2;
    i2cRegisters(i2cBaseAddress)   <= 16#FF#;
    i2cRegisters(i2cBaseAddress+1) <= 16#FF#;
    i2cWrite <= '0';
    i2cSend <= '1', '0' after 1 ns;
    wait until i2cDone = '1';
                                                               -- ask for status
    wait for 3 ms - now;
    write(output,
      "Asking for status" &
      " at time " & integer'image(now/1 us) & " us" &
      lf &
      "  Should be FFFFh (reached)" &
      lf & lf
    );
    i2cBaseAddress <= baseAddress; wait for 0 ns;
    i2cDataLength <= 2;
    i2cRegisters(i2cBaseAddress)   <= 16#FF#;
    i2cRegisters(i2cBaseAddress+1) <= 16#FF#;
    i2cWrite <= '0';
    i2cSend <= '1', '0' after 1 ns;
    wait until i2cDone = '1';
                                                             -- ask for position
    wait for 2*i2cPeriod;
    write(output,
      "Asking for position" &
      " at time " & integer'image(now/1 us) & " us" &
      lf & lf
    );
    i2cBaseAddress <= baseAddress + 2; wait for 0 ns;
    i2cDataLength <= 2;
    i2cRegisters(i2cBaseAddress)   <= 16#FF#;
    i2cRegisters(i2cBaseAddress+1) <= 16#FF#;
    i2cWrite <= '0';
    i2cSend <= '1', '0' after 1 ns;
    wait until i2cDone = '1';
                                                              -- send full angle
    wait for 300*i2cPeriod;
    write(output,
      "Sending turn control to full angle" &
      " at time " & integer'image(now/1 us) & " us" &
      lf & lf
    );
    i2cBaseAddress <= baseAddress + 2; wait for 0 ns;
    i2cDataLength <= 2;
    i2cRegisters(i2cBaseAddress)   <= angleMaxValue mod 2**i2cDataBitNb;
    i2cRegisters(i2cBaseAddress+1) <= angleMaxValue / 2**i2cDataBitNb;
    i2cWrite <= '1';
    i2cSend <= '1', '0' after 1 ns;
                                                              -- send zero angle
    wait for 3 ms;
    write(output,
      "Sending turn control to angle zero" &
      " at time " & integer'image(now/1 us) & " us" &
      lf & lf
    );
    i2cBaseAddress <= baseAddress + 2; wait for 0 ns;
    i2cDataLength <= 2;
    i2cRegisters(i2cBaseAddress)   <= 0;
    i2cRegisters(i2cBaseAddress+1) <= 0;
    i2cWrite <= '1';
    i2cSend <= '1', '0' after 1 ns;
                                                -- assert end contact from slave
    wait for 4 ms;
    write(output,
      "Sending end contact ON" &
      " at time " & integer'image(now/1 us) & " us" &
      lf & lf
    );
    wait for 10*i2cPeriod;
    i2cBaseAddress <= baseAddress + 4; wait for 0 ns;
    i2cDataLength <= 1;
    i2cRegisters(i2cBaseAddress)   <= 16#FE#;
    i2cWrite <= '0';
    i2cSend <= '1', '0' after 1 ns;
    wait until i2cDone = '1';
                                              -- deassert end contact from slave
    wait for 10*i2cPeriod;
    write(output,
      "Sending end contact OFF" &
      " at time " & integer'image(now/1 us) & " us" &
      lf & lf
    );
    i2cBaseAddress <= baseAddress + 4; wait for 0 ns;
    i2cDataLength <= 1;
    i2cRegisters(i2cBaseAddress)   <= 16#FF#;
    i2cWrite <= '0';
    i2cSend <= '1', '0' after 1 ns;
    wait until i2cDone = '1';
                                                              -- send half angle
    wait for 100*i2cPeriod;
    write(output,
      "Sending turn control to half angle" &
      " at time " & integer'image(now/1 us) & " us" &
      lf & lf
    );
    i2cBaseAddress <= baseAddress + 2; wait for 0 ns;
    i2cDataLength <= 2;
    i2cRegisters(i2cBaseAddress)   <= (angleMaxValue+1)/2 mod 2**i2cDataBitNb;
    i2cRegisters(i2cBaseAddress+1) <= (angleMaxValue+1)/2 / 2**i2cDataBitNb;
    i2cWrite <= '1';
    i2cSend <= '1', '0' after 1 ns;
                                               -- assert end contact from master
    wait for 500*i2cPeriod;
    write(output,
      "Sending end contact ON" &
      " at time " & integer'image(now/1 us) & " us" &
      lf & lf
    );
    wait for 10*i2cPeriod;
    i2cBaseAddress <= orientationAddress; wait for 0 ns;
    i2cDataLength <= 1;
    i2cRegisters(orientationAddress)   <= 16#08#;
    i2cWrite <= '1';
    i2cSend <= '1', '0' after 1 ns;
    wait until i2cDone = '1';
                                             -- deassert end contact from master
    wait for 10*i2cPeriod;
    write(output,
      "Sending end contact OFF" &
      " at time " & integer'image(now/1 us) & " us" &
      lf & lf
    );
    i2cBaseAddress <= orientationAddress; wait for 0 ns;
    i2cDataLength <= 1;
    i2cRegisters(i2cBaseAddress) <= 16#00#;
    i2cWrite <= '1';
    i2cSend <= '1', '0' after 1 ns;
    wait until i2cDone = '1';
                                                            -- end of simulation
    wait for 1 ms;
    assert false
      report "End of simulation"
      severity failure;
    wait;
  end process;


  --============================================================================
                                                               -- i2c send frame
  sendI2cFrame: process
  begin
    i2cDone <= '0';
    i2cSendWord <= '0';
    sCl <= 'H';
    sDa <= 'H';
    wait until rising_edge(i2cSend);
                                                              -- start condition
    sDa <= '0', 'H' after i2cPeriod;
    wait for i2cPeriod/2;
                                                            -- send chip address
    i2cDataOut <= to_unsigned(chipAddress/2, i2cDataOut'length-2) & '0' & '1';
    i2cSendWord <= '1', '0' after 1 ns;
    wait until falling_edge(i2cWordDone);
                                                        -- send register address
    i2cDataOut <= to_unsigned(i2cBaseAddress, i2cDataOut'length-1) & '1';
    i2cSendWord <= '1', '0' after 1 ns;
    wait until falling_edge(i2cWordDone);
                                                                 -- write access
    if i2cWrite = '1' then
                                                              -- send data bytes
      for wordIndex in i2cBaseAddress to i2cBaseAddress+i2cDataLength-1 loop
        i2cDataOut <= unsigned(
          to_signed(i2cRegisters(wordIndex), i2cDataOut'length-1) & '1'
        );
        if (i2cWrite = '0') and (wordIndex = i2cBaseAddress+i2cDataLength-1) then
          i2cDataOut(0) <= '0';
        end if;
        i2cSendWord <= '1', '0' after 1 ns;
        wait until falling_edge(i2cWordDone);
      end loop;
                                                                  -- read access
    else
                                                              -- start condition
      wait for 4*i2cPeriod;
      sDa <= '0', 'H' after i2cPeriod;
      wait for i2cPeriod/2;
                                                            -- send chip address
      i2cDataOut <= to_unsigned(chipAddress/2, i2cDataOut'length-2) & not(i2cWrite) & '1';
      i2cSendWord <= '1', '0' after 1 ns;
      wait until falling_edge(i2cWordDone);
                                                              -- read data bytes
      for wordIndex in i2cBaseAddress to i2cBaseAddress+i2cDataLength-1 loop
        i2cDataOut <= unsigned(
          to_signed(i2cRegisters(wordIndex), i2cDataOut'length-1) & '1'
        );
        if wordIndex < i2cBaseAddress+i2cDataLength-1 then
          i2cDataOut(0) <= '0';
        end if;
        i2cSendWord <= '1', '0' after 1 ns;
        wait until falling_edge(i2cWordDone);
      end loop;
    end if;
                                                               -- stop condition
    sDa <= '0';
    wait for i2cPeriod/2;
    sCl <= 'H';
    i2cDone <= '1';
    wait for i2cPeriod/2;
  end process sendI2cFrame;

  ------------------------------------------------------------------------------
                                                    -- i2c send and receive byte
  sendI2cByte: process
    variable i2cReadData: unsigned(i2cBitNb-1-1 downto 0);
  begin
    i2cWordDone <= '0';
    sCl <= 'H' after i2cPeriod/4;
    sDa <= 'H';
    wait until rising_edge(i2cSendWord);
                                                                    -- send byte
    for bitIndex in i2cDataOut'range loop
      sCl <= '0';
      wait for i2cPeriod/4;
      if i2cDataOut(bitIndex) = '0' then sDa <= '0'; else sDa <= 'H'; end if;
      wait for i2cPeriod/4;
      sCl <= 'H';
      i2cReadData := shift_left(i2cReadData, 1);
      i2cReadData(0) := to_X01(sDa);
      wait for i2cPeriod/2;
    end loop;
    if i2cWrite = '0' then
      i2cDataIn <= to_integer(shift_right(i2cReadData, 1));
    end if;
    sCl <= '0';
    wait for i2cPeriod;
    i2cWordDone <= '1';
    wait for 1 ns;
  end process sendI2cByte;

END ARCHITECTURE test;
